/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_debug_console.h"
#include "board.h"
#include "math.h"
#include "fsl_fxos.h"
#include "fsl_i2c.h"
#include "fsl_ftm.h"
#include "fsl_port.h"
#include "fsl_common.h"
#include "pin_mux.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* The Flextimer instance/channel used for board */
#define BOARD_FTM_BASEADDR FTM0
#define BOARD_FIRST_TIMER_CHANNEL 0U
#define BOARD_SECOND_TIMER_CHANNEL 1U
#define BOARD_FIRST_CHANNEL_INT kFTM_Chnl0InterruptEnable
#define BOARD_SECOND_CHANNEL_INT kFTM_Chnl1InterruptEnable
#define BOARD_FTM_IRQ_HANDLER_FUNC FTM0_IRQHandler
#define BOARD_FTM_IRQ_ID FTM0_IRQn
/* Get source clock for TPM driver */
#define BOARD_TIMER_SOURCE_CLOCK CLOCK_GetFreq(kCLOCK_BusClk)
/* I2C source clock */
#define ACCEL_I2C_CLK_SRC I2C0_CLK_SRC
#define I2C_BAUDRATE 100000U
/* Upper bound and lower bound angle values */
#define ANGLE_UPPER_BOUND 85U
#define ANGLE_LOWER_BOUND 5U

#define RONG 1
void pinMuxFTM0(void);
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
static void Board_UpdatePwm(uint16_t x, uint16_t y);
/*******************************************************************************
 * Variables
 ******************************************************************************/
i2c_master_handle_t g_MasterHandle;
volatile int16_t g_xAngle = 0;
volatile int16_t g_yAngle = 0;
/* FXOS device address */
const uint8_t g_accel_address[] = {0x1CU, 0x1DU, 0x1EU, 0x1FU};

/*******************************************************************************
 * Code
 ******************************************************************************/
/* Initialize timer module */
static void Timer_Init(void)
{
    ftm_config_t ftmInfo;

    ftm_chnl_pwm_signal_param_t ftmParam[4];
   // ftm_chnl_pwm_signal_param_t ftmParam[2];
    /* Configure ftm params with frequency 24kHZ */
    //FTM_CH4 and FTM_CH5 pair
    ftmParam[0].chnlNumber = (ftm_chnl_t)BOARD_FIRST_TIMER_CHANNEL;  //FTM0_CH0
    ftmParam[0].level = kFTM_HighTrue; kFTM_LowTrue;
    ftmParam[0].dutyCyclePercent = 50U;
    ftmParam[0].firstEdgeDelayPercent = 0U;

    //FTM_CH6 and FTM_CH7 channel pair
    ftmParam[1].chnlNumber = (ftm_chnl_t)BOARD_SECOND_TIMER_CHANNEL; //FTM0_CH1
    ftmParam[1].level = kFTM_HighTrue;
    ftmParam[1].dutyCyclePercent = 50U;
    ftmParam[1].firstEdgeDelayPercent = 0U;

    //FTM_CH4 and FTM_CH5 pair
    ftmParam[2].chnlNumber = (ftm_chnl_t)2;  //FTM0_CH2
    ftmParam[2].level = kFTM_LowTrue;
    ftmParam[2].dutyCyclePercent = 50U;
    ftmParam[2].firstEdgeDelayPercent = 0U;

    //FTM_CH6 and FTM_CH7 channel pair
    ftmParam[3].chnlNumber = (ftm_chnl_t)3; //FTM0_CH3
    ftmParam[3].level = kFTM_LowTrue;
    ftmParam[3].dutyCyclePercent = 50U;
    ftmParam[3].firstEdgeDelayPercent = 0U;
    /*
     * ftmInfo.prescale = kFTM_Prescale_Divide_1;
     * ftmInfo.bdmMode = kFTM_BdmMode_0;
     * ftmInfo.pwmSyncMode = kFTM_SoftwareTrigger;
     * ftmInfo.reloadPoints = 0;
     * ftmInfo.faultMode = kFTM_Fault_Disable;
     * ftmInfo.faultFilterValue = 0;
     * ftmInfo.deadTimePrescale = kFTM_Deadtime_Prescale_1;
     * ftmInfo.deadTimeValue = 0;
     * ftmInfo.extTriggers = 0;
     * ftmInfo.chnlInitState = 0;
     * ftmInfo.chnlPolarity = 0;
     * ftmInfo.useGlobalTimeBase = false;
     */
    FTM_GetDefaultConfig(&ftmInfo);
    /* Initialize FTM module */
    FTM_Init(BOARD_FTM_BASEADDR, &ftmInfo);

    FTM_EnableInterrupts(BOARD_FTM_BASEADDR,
                         BOARD_FIRST_CHANNEL_INT | BOARD_SECOND_CHANNEL_INT | kFTM_TimeOverflowInterruptEnable);
    NVIC_EnableIRQ(BOARD_FTM_IRQ_ID);
   //Rong wrote:
    // FTM_SetupPwm(BOARD_FTM_BASEADDR, ftmParam, 2U, kFTM_EdgeAlignedPwm, 24000U, BOARD_TIMER_SOURCE_CLOCK);
    FTM_SetupPwm(BOARD_FTM_BASEADDR, ftmParam, 4U, kFTM_EdgeAlignedPwm, 24000U, BOARD_TIMER_SOURCE_CLOCK);
  //  FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR,BOARD_FIRST_TIMER_CHANNEL,kFTM_EdgeAlignedPwm,50);
   // FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR,BOARD_SECOND_TIMER_CHANNEL,kFTM_EdgeAlignedPwm,50);
   // FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR,2,kFTM_EdgeAlignedPwm,50);
  //  FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR,3,kFTM_EdgeAlignedPwm,50);
    //   FTM_SetComplementaryEnable(BOARD_FTM_BASEADDR,ftmParam[0].chnlNumber, 1);
 //   FTM_SetComplementaryEnable(BOARD_FTM_BASEADDR,ftmParam[1].chnlNumber, 1);
    FTM_StartTimer(BOARD_FTM_BASEADDR, kFTM_SystemClock);
}

void BOARD_FTM_IRQ_HANDLER_FUNC(void)
{
    uint32_t status = FTM_GetStatusFlags(BOARD_FTM_BASEADDR);

    if (status & kFTM_TimeOverflowFlag)
    {
       // Board_UpdatePwm(g_xAngle, g_yAngle);
        if (g_xAngle)
        {
            LED_GREEN_ON();
        }
        else
        {
            LED_GREEN_OFF();
        }
        if (g_yAngle)
        {
            LED_RED_ON();
        }
        else
        {
            LED_RED_OFF();
        }
        FTM_ClearStatusFlags(BOARD_FTM_BASEADDR, kFTM_TimeOverflowFlag);
    }
    if (status & kFTM_Chnl0Flag)
    {
        LED_GREEN_OFF();
        FTM_ClearStatusFlags(BOARD_FTM_BASEADDR, kFTM_Chnl0Flag);
    }
    if (status & kFTM_Chnl1Flag)
    {
        LED_RED_OFF();
        FTM_ClearStatusFlags(BOARD_FTM_BASEADDR, kFTM_Chnl1Flag);
    }
}

/* Update the duty cycle of an active pwm signal */
static void Board_UpdatePwm(uint16_t x, uint16_t y)
{
    /* Start PWM mode with updated duty cycle */
    FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR, (ftm_chnl_t)BOARD_FIRST_TIMER_CHANNEL, kFTM_EdgeAlignedPwm, x);
    FTM_UpdatePwmDutycycle(BOARD_FTM_BASEADDR, (ftm_chnl_t)BOARD_SECOND_TIMER_CHANNEL, kFTM_EdgeAlignedPwm, y);
    /* Software trigger to update registers */
    FTM_SetSoftwareTrigger(BOARD_FTM_BASEADDR, true);
}

int main(void)
{
    fxos_handle_t fxosHandle = {0};
    fxos_data_t sensorData = {0};
    i2c_master_config_t i2cConfig = {0};
    uint32_t i2cSourceClock = 0;
    int16_t xData = 0;
    int16_t yData = 0;
    uint8_t i = 0;
    uint8_t regResult = 0;
    uint8_t array_addr_size = 0;
    bool foundDevice = false;

    i2cSourceClock = CLOCK_GetFreq(ACCEL_I2C_CLK_SRC);
    fxosHandle.base = BOARD_ACCEL_I2C_BASEADDR;
    fxosHandle.i2cHandle = &g_MasterHandle;

    /* Board pin, clock, debug console init */
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    /*
     * i2cConfig.baudRate_Bps = 100000U;
     * i2cConfig.enableHighDrive = false;
     * i2cConfig.enableStopHold = false;
     * i2cConfig.glitchFilterWidth = 0U;
     * i2cConfig.enableMaster = true;
     */
    I2C_MasterGetDefaultConfig(&i2cConfig);
    I2C_MasterInit(BOARD_ACCEL_I2C_BASEADDR, &i2cConfig, i2cSourceClock);
    I2C_MasterTransferCreateHandle(BOARD_ACCEL_I2C_BASEADDR, &g_MasterHandle, NULL, NULL);

    /* Find sensor devices */
    array_addr_size = sizeof(g_accel_address) / sizeof(g_accel_address[0]);
    for (i = 0; i < array_addr_size; i++)
    {
        fxosHandle.xfer.slaveAddress = g_accel_address[i];
        if (FXOS_ReadReg(&fxosHandle, WHO_AM_I_REG, &regResult, 1) == kStatus_Success)
        {
            foundDevice = true;
            break;
        }
        if ((i == (array_addr_size - 1)) && (!foundDevice))
        {
            PRINTF("\r\nDo not found sensor device\r\n");
            while (1)
            {
            };
        }
    }

    /* Init accelerometer sensor */
    FXOS_Init(&fxosHandle);
    /* Init timer */
    pinMuxFTM0();
    Timer_Init();

    /* Print a note to terminal */
    PRINTF("\r\nWelcome to BUBBLE example\r\n");
    PRINTF("\r\nYou will see the change of LED brightness when change angles of board\r\n");

    /* Main loop. Get sensor data and update duty cycle */
    while (1)
    {
        /* Get new accelerometer data. */
        FXOS_ReadSensorData(&fxosHandle, &sensorData);

        /* Get the X and Y data from the sensor data structure.fxos_data */
        xData = (int16_t)((uint16_t)((uint16_t)sensorData.accelXMSB << 8) | (uint16_t)sensorData.accelXLSB);
        yData = (int16_t)((uint16_t)((uint16_t)sensorData.accelYMSB << 8) | (uint16_t)sensorData.accelYLSB);

        /* Convert raw data to angle (normalize to 0-90 degrees). No negative angles. */
        g_xAngle = (int16_t)floor((double)xData * 0.011);
        if (g_xAngle < 0)
        {
            g_xAngle *= -1;
        }
        g_yAngle = (int16_t)floor((double)yData * 0.011);
        if (g_yAngle < 0)
        {
            g_yAngle *= -1;
        }
        /* Update angles to turn on LEDs when angles ~ 90 */
        if (g_xAngle > ANGLE_UPPER_BOUND)
        {
            g_xAngle = 100;
        }
        if (g_yAngle > ANGLE_UPPER_BOUND)
        {
            g_yAngle = 100;
        }
        /* Update angles to turn off LEDs when angles ~ 0 */
        if (g_xAngle < ANGLE_LOWER_BOUND)
        {
            g_xAngle = 0;
        }
        if (g_yAngle < ANGLE_LOWER_BOUND)
        {
            g_yAngle = 0;
        }
        /* Print out the raw accelerometer data. */
        PRINTF("x= %d y = %d\r\n", xData, yData);
    }
}



void pinMuxFTM0(void)
{

    /* Ungate the port clock */
    CLOCK_EnableClock(kCLOCK_PortC);
    /* PTC1/FTM0_CH0 Affects PORTC_PCR1 register */
    PORT_SetPinMux(PORTC, 1U, kPORT_MuxAlt4);
    /* Affects PORTB_PCR17 register */
    //PTC2/FTM0_CH1
    PORT_SetPinMux(PORTC, 2U, kPORT_MuxAlt4);
    //PTC3/FTM0_CH2
    PORT_SetPinMux(PORTC, 3U, kPORT_MuxAlt4);
    /* Affects PORTB_PCR17 register */
    //PTC4/FTM0_CH3
    PORT_SetPinMux(PORTC, 4U, kPORT_MuxAlt4);
}
